clear mata
mata: 

// CRRA separable utility
real scalar utility(real scalar c, real scalar h, real scalar sigma) {

	if(c<=0) return (-9999999)
	
	if(abs(sigma-1)<0.00001) return ( ln(c) + ln(h)  )
	
	return( (c^(1-sigma) - 1) / (1-sigma) + (h^(1-sigma) - 1) / (1-sigma) )
}

// First derivative of CRRA separable utility
real scalar dutility(real scalar c, real scalar h, real scalar sigma) {

	if(c<=0) return (1)
	
	if(abs(sigma-1)<0.00001) return (1/c)
	
	return( c^(-sigma) )
}

// Second derivative of CRRA separable utility
real scalar d2utility(real scalar c, real scalar h, real scalar sigma) {

	if(c<=0) return (0)
	
	if(abs(sigma-1)<0.00001) return (-1/(c*c))
	
	return( -sigma*(c^(-sigma-1)) )
}

// Solve for ex ante value (might include financial insurance, or might not)
void solver_iv(todo, wtp, c_s, c_w, h_Delta, h_w, prob, EU, sigma, f, g, H) {

	num_sick_states = cols(prob)
	prob_well = 1 - rowsum(prob)

	func = EU - prob_well*utility(c_w-wtp,h_w,sigma)
	dfunc =    prob_well*dutility(c_w-wtp,h_w,sigma)
	d2func = -prob_well*d2utility(c_w-wtp,h_w,sigma)
	
	for(i = 1; i<=num_sick_states; i++) {
	
		func = func -       prob[1,i]*utility(c_s[1,i]-wtp,h_Delta[1,i],sigma)
		dfunc = dfunc +    prob[1,i]*dutility(c_s[1,i]-wtp,h_Delta[1,i],sigma)
		d2func = d2func - prob[1,i]*d2utility(c_s[1,i]-wtp,h_Delta[1,i],sigma)
	}	
	
	// Function being maximized by solver:
	f = -(func^2)
	
	if (todo>=1) {
	
		// First derivative of f
		g[1] = -2*func*( dfunc )
		
		if (todo==2) {
		
			// Second derivative of f
			H[1,1] = -2*( func*d2func + dfunc*dfunc )
			_makesymmetric(H)
		}
	}

}
	
// Solve for ex post ("traditional"/"conventional") value
void solver_rfv(todo, ce, y, h,EU,sigma, f, g, H) {

	func   = EU - utility(y-ce,h,sigma)
	dfunc  =     dutility(y-ce,h,sigma)
	d2func =   -d2utility(y-ce,h,sigma)
	
	// Function being maximized by solver:
	f = -(func^2)
	
	if (todo>=1) {
	
		// First derivative of f
		g[1] = -2*func*( dfunc )
		
		if (todo==2) {
		
			// Second derivative of f
			H[1,1] = -2*( func*d2func + dfunc*dfunc )
			_makesymmetric(H)
		}
	}

}

// This function sets the optimization parameters and calls the solver functions above. Following Ehrlich-Becker, insurance value ("iv") is sometimes referred to as self-insurance value ("siv"), and financial insurance value sometimes referred to as market-insurance value ("miv").
void estimate_wtp(real scalar scale_factor, string newvarname_rfv, string newvarname_iv, string newvarname_fiv) {

	// Ensure that h_s, y_s, Price, and prob are aordered. Note: Must be executed be before the st_view() commands, or those pointers will be wrong
	stata("unab vars: *")
	stata("order h_Delta* h_s* y_s* Price* prob*, sequential")
	
	// newvarname will store the result
	stata("confirm new var " + newvarname_rfv)
	st_addvar("double",newvarname_rfv);
	st_view(rfv=.,.,newvarname_rfv)	
	
	stata("confirm new var " + newvarname_iv)
	st_addvar("double",newvarname_iv);
	st_view(siv=.,.,newvarname_iv)
	
	stata("confirm new var " + newvarname_fiv)
	st_addvar("double",newvarname_fiv);
	st_view(miv=.,.,newvarname_fiv)
	
	// Vars that don't depend on the number of sick states
	st_view(hw_vec=.,.,"h_w")
	yw_vec=st_data(.,"y_w")
	st_view(sigma_vec=.,.,"sigma")
	
	// Vars that do depend on the number of sick states. Note: yw, ys, and P are data copies, not views, b/c they at some point get divided by scale factor.
	hs_vec=st_data(.,"h_s*")
	hDelta_vec=st_data(.,"h_Delta*")
	ys_vec=st_data(.,"y_s*")
	P_vec=st_data(.,"Price*")
	st_view(prob_vec=.,.,"prob*")
	
	// Calculate prob of well state
	stata("confirm new var prob_well")
	st_addvar("double","prob_well");
	st_view(prob_well_vec=.,.,"prob_well")
	prob_well_vec[.,1] = J(rows(sigma_vec),1,1)-rowsum(prob_vec)
	stata("assert inrange(prob_well,0,1)")
	
	num_sick_states = cols(prob_vec)
	stata(`"di "number of sick states: ""' + strofreal(num_sick_states))
	
	if(num_sick_states>1) {
		for(i=1;i<=num_sick_states; i++) {
			stata("confirm new var _" + newvarname_rfv + strofreal(i))
			st_addvar("double","_"+newvarname_rfv + strofreal(i));
		}
	}
	
	// Rescale prices, income and health if there are numerical problems
	P_vec = P_vec :/ scale_factor
	ys_vec = ys_vec :/ scale_factor
	yw_vec = yw_vec :/ scale_factor
	
	hs_vec = hs_vec :/ scale_factor
	hw_vec = hw_vec :/ scale_factor
	hDelta_vec = hDelta_vec :/ scale_factor
	
	// For each row, calculate traditional ex post value, total insurance value, and financial spending risk
	for(rownum=1; rownum<=rows(sigma_vec); rownum++) {
	
		h_w =      hw_vec[rownum,1]
		y_w =      yw_vec[rownum,1]
		sigma =    sigma_vec[rownum,1]
		prob_well = prob_well_vec[rownum,1]		
		
		premiums = 0
		EU_sick = 0
		for(colnum_j = 1; colnum_j<=num_sick_states; colnum_j++) {
		
			h_Delta =  hDelta_vec[rownum,colnum_j]	
			h_s =      hs_vec[rownum,colnum_j]
			y_s =      ys_vec[rownum,colnum_j]			
			prob_i =   prob_vec[rownum,colnum_j]

			EU_sick = EU_sick + prob_i * utility(y_s,h_s,sigma)
			
			premiums = premiums +  prob_i * P_vec[rownum,colnum_j]
		}		
		EU = EU_sick  + prob_well*utility(y_w,h_w,sigma)

		cs_siv = J(1,num_sick_states,0)
		cs_miv = J(1,num_sick_states,0)		
		for(colnum_j = 1; colnum_j<=num_sick_states; colnum_j++) {
			cs_siv[1,colnum_j] = ys_vec[rownum,colnum_j] - P_vec[rownum,colnum_j]
			cs_miv[1,colnum_j] = ys_vec[rownum,colnum_j] - premiums
		}
		
		cw_siv = y_w
		cw_miv = y_w - premiums

		// 1. Calculate total insurance value
		S = optimize_init()
		optimize_init_evaluator(S, &solver_iv())
		optimize_init_evaluatortype(S, "d2")
	
		optimize_init_which(S,  "max" )
		optimize_init_tracelevel(S,"none")
	
		optimize_init_params(S, 0)
		optimize_init_argument(S,1,cs_siv)
		optimize_init_argument(S,2,cw_siv)
		optimize_init_argument(S,3,hDelta_vec[rownum,.])
		optimize_init_argument(S,4,h_w)
		optimize_init_argument(S,5,prob_vec[rownum,.])
		optimize_init_argument(S,6,EU)
		optimize_init_argument(S,7,sigma)
		wtp_siv = optimize(S)
	
		// 2. Calculate financial spending risk
		optimize_init_evaluator(S, &solver_iv())		
		optimize_init_params(S, 0)
		optimize_init_argument(S,1,cs_miv)
		optimize_init_argument(S,2,cw_miv)		
		wtp_miv = optimize(S)
		
		// 3. Calculate ex post value (aka "risk-free", "conventional", "traditional" value)
		RFV = 0
		for(colnum_j = 1; colnum_j<=num_sick_states; colnum_j++) {
		
			h_Delta =  hDelta_vec[rownum,colnum_j]	
			h_s =      hs_vec[rownum,colnum_j]
			y_s =      ys_vec[rownum,colnum_j]
			P_i =      P_vec[rownum,colnum_j]	
			prob_i =   prob_vec[rownum,colnum_j]

			u_sick_i = utility(y_s,h_s,sigma)

			S_rfv = optimize_init()
			optimize_init_evaluator(S_rfv, &solver_rfv())
			optimize_init_evaluatortype(S_rfv, "d2")

			optimize_init_which(S_rfv,  "max" )
			optimize_init_tracelevel(S_rfv,"none")
		
			optimize_init_params(S_rfv, 0)
			optimize_init_argument(S_rfv,1,y_s)
			optimize_init_argument(S_rfv,2,h_Delta)
			optimize_init_argument(S_rfv,3,u_sick_i)
			optimize_init_argument(S_rfv,4,sigma)	
			
			WTP = optimize(S_rfv)
			RFV = RFV + prob_i*(WTP-P_i)

			if(num_sick_states>1) {
				v = "_"+newvarname_rfv + strofreal(colnum_j)
				st_view(rfv_view=.,.,v)
				tmp = prob_i*(WTP-P_i)*scale_factor
				if (abs(tmp)<0.01) tmp=0
				rfv_view[rownum,1] = tmp
			}
		}

		// Save results
		rfv[rownum,1] = RFV*scale_factor
		siv[rownum,1] = (wtp_siv-RFV)*scale_factor
		miv[rownum,1] = (wtp_miv-wtp_siv)*scale_factor

	}
	
	stata("drop prob_well")
	var_order = st_local("vars")
	stata("order " + var_order)
}
end

** EOF
